@@ -98,7 +98,8 @@ def get_src_binaries(options, src):
 
 
 def is_toolchain(pkg):
-    """Is this tool incapable of cross-building pkg?"""
+    """Is this package provided by the cross-toolchain?"""
+    """We really ought to check if they actually are installed or equived rather than assuming""" 
     return ('libc6' in pkg or 'lib32gcc' in pkg or 'lib64gcc' in pkg or
             'libgcc' in pkg or 'libgcj' in pkg or 'libstdc++' in pkg or
             'multilib' in pkg or
@@ -141,6 +142,9 @@ explicit_requests = set()
 all_srcs = {}
 depends = {}
 needs_build = {}
+dot_relationships = dict()
+needs_install = {}
+install_version = {}
 
 def filter_arch(deps, arch):
     """Filter out (build-)dependencies that do not apply to arch."""
@@ -171,17 +175,27 @@ def filter_arch(deps, arch):
             new_deps.append(new_or_deps)
     return new_deps
 
-def should_expand(parent, pkg):
+def should_expand(options, parent, pkg):
     if pkg in explicit_requests:
         # We're going to build it anyway because it was requested on the
         # command line, so we might as well sequence it properly.
         return True
+     # if options.generate_graph:
+        # If generating non-compact graphs we want to report _real_ deps
+        # and not just ones that  xdeb approves of - however this just explodes dot
+        # disable for now
+        # return True;
     return (is_crossable(pkg) and not is_toolchain(pkg) and
             not need_loop_break(parent, pkg))
 
-def expand_depends(options, pkg, depth, builddep_depth,
+def expand_depends(current_graph, options, pkg, depth, builddep_depth, 
                    parent=None, parent_binary=None):
     """Recursively expand (build-)dependencies of pkg."""
+    # Get control data from apt or local dir
+    # depth and builddep-depth are current level of recursion
+    # optionally output results in human readable (debug option), 
+    #  or dot format (generate-graph option)
+    # 
     src = get_src_name(options, pkg)
     if not src:
         # Is it already a source package name?
@@ -208,8 +222,20 @@ def expand_depends(options, pkg, depth, builddep_depth,
         if options.debug:
             print "Did not find a source record for %s" % src 
         return
+    if options.debug:
+        print "expand_depends; parent=%s, package=%s, source=%s, depth=%s, builddepth=%s" % (parent, pkg, src, depth, builddep_depth)
+        print parent
 
     if parent is not None and parent != src:
+        if options.generate_graph:
+            if options.generate_compact_graph:
+                rel = '"%s" -> "%s";' % ( parent, src)
+                dot_relationships[current_graph].append(rel)
+            else:
+                style = '"bin-%s" [label="%s",shape=box];' % ( pkg, pkg )
+                rel = '"%s" -> "bin-%s" -> "%s";' % ( parent, pkg, src)
+                dot_relationships[current_graph].append(style)
+                dot_relationships[current_graph].append(rel)
         if options.debug:
             if parent_binary is not None:
                 print "%s%s (%s) -> %s (%s)" % (' ' * depth,
@@ -239,33 +265,38 @@ def expand_depends(options, pkg, depth, builddep_depth,
                 # break this cycle
                 continue
             bd_pkg = builddep[0]['name']
-            if should_expand(src, bd_pkg):
-                expand_depends(options, bd_pkg, depth + 1, builddep_depth + 1,
+            if should_expand(options, src, bd_pkg):
+                expand_depends(current_graph, options, bd_pkg, depth + 1, builddep_depth + 1,
                                parent=src)
 
-    for binary in get_src_binaries(options, src):
-        if not is_crossable(binary) or is_toolchain(binary):
-            continue
-        parsed_binary = get_pkg_record(options, binary)
-        deps = parsed_binary.relations['pre-depends']
-        deps.extend(parsed_binary.relations['depends'])
-        filter_arch(deps, options.architecture)
-        for dep in deps:
-            for or_dep in dep:
-                # Check should_expand twice, once for the virtual package
-                # name and once for the real package name (which may be
-                # different).
-                if not should_expand(src, or_dep['name']):
+    # Find the (install-)dependencies of the source package to ensure that 
+    # what is built can be installed. However we don't want to do this
+    # for a top-level package as that is not being installed (yet)
+    if depth:
+        for binary in get_src_binaries(options, src):
+            if not options.generate_graph:
+                if (not is_crossable(binary)) or is_toolchain(binary):
                     continue
-                # TODO version handling?
-                real_dep = get_real_pkg(options, or_dep['name'])
-                if (real_dep is not None and
-                    get_src_name(options, real_dep) is not None):
-                    if should_expand(src, real_dep):
-                        expand_depends(options, real_dep, depth + 1,
-                                       builddep_depth,
-                                       parent=src, parent_binary=binary)
-                    break
+            parsed_binary = get_pkg_record(options, binary)
+            deps = parsed_binary.relations['pre-depends']
+            deps.extend(parsed_binary.relations['depends'])
+            filter_arch(deps, options.architecture)
+            for dep in deps:
+                for or_dep in dep:
+                    # Check should_expand twice, once for the virtual package
+                    # name and once for the real package name (which may be
+                    # different).
+                    if not should_expand(options, src, or_dep['name']):
+                        continue
+                    # TODO version handling?
+                    real_dep = get_real_pkg(options, or_dep['name'])
+                    if (real_dep is not None and
+                        get_src_name(options, real_dep) is not None):
+                        if should_expand(options, src, real_dep):
+                            expand_depends(current_graph, options, real_dep, depth + 1,
+                                           builddep_depth,
+                                           parent=src, parent_binary=binary)
+                        break
 
 def mark_needs_build(options, src, force=False):
     """Decide whether a package needs to be (re)built."""
@@ -285,6 +316,9 @@ def mark_needs_build(options, src, force=False):
                     print "%s already built at version %s" % (src,
                                                               built_vers[-1])
                 newer = False
+                install_version[src] = built_vers[-1]
+                if check_if_src_needs_install(src, built_vers[-1], options):
+                    needs_install[src] = True
             else:
                 newer = True
         else:
@@ -484,6 +518,7 @@ def native_import(options, src):
             except StopIteration:
                 break
 
+    #fake up a changes file
     changes = '%s_%s_%s.changes' % (src, ver_no_epoch, options.architecture)
     changes_file = open(os.path.join(options.builddirs[0], changes), 'w')
     print >>changes_file, 'Version: %s\nFake: yes' % ver
@@ -597,6 +632,9 @@ def build(options, src, ver):
 
     outdir = os.path.normpath(os.path.join(
         tree.get_src_directory(options, src), '..'))
+    install_built_pkg(src, ver_no_epoch, options, outdir)
+    
+def install_built_pkg(src, ver_no_epoch, options, outdir, force=True):
     changes = '%s_%s_%s.changes' % (src, ver_no_epoch, options.architecture)
     build_log = '%s_%s_%s.build' % (src, ver_no_epoch, options.architecture)
     debs = get_output(['dcmd', '--deb', changes], cwd=outdir).splitlines()
@@ -646,7 +684,30 @@ def build(options, src, ver):
 
     update_apt_repository(options)
 
-
+def check_if_src_needs_install(src, ver, options):
+    ver_no_epoch = re_no_epoch.sub('', str(install_version[src]))
+    changes = '%s_%s_%s.changes' % (src, ver_no_epoch, options.architecture)
+    debs = get_output(['dcmd', '--deb', changes], cwd=options.destdir).splitlines()
+    install_reqd = False
+    for deb in debs:
+        pkg = re_package.sub(r"\1", deb)
+        if pkg in all_builddeps:
+            print "builddep: %s" % pkg
+            pkg_cross = "%s-%s-cross" % (pkg, options.architecture)
+            try:
+                if not aptutils.cache[pkg].is_installed:
+                    if options.debug:
+                        print ("% is not installed" % pkg)
+                    install_reqd = True
+                if not aptutils.cache[pkg_cross].is_installed:
+                    if options.debug:
+                        print ("% is not installed" % pkg)
+                    install_reqd = True
+            except:
+                print "failed to find pkg in cache:" + pkg
+                install_reqd = True
+    return install_reqd
+    
 def main():
     usage = '%prog [options] package ...'
     parser = optparse.OptionParser(usage=usage)
@@ -715,6 +776,13 @@ def main():
     parser.add_option('--no-convert-install', dest='convert_install',
                       action='store_false', default=True,
                       help="don't install packages after cross-conversion")
+    parser.add_option('--generate-graph', default=False, action='store_true',
+                      dest='generate_graph', help='Generates a dot file that '
+                                'can be drawn by an app like GraphViz'
+                                'WARNING: WILL UNSET --debug')
+    parser.add_option('--generate-compact-graph', dest='generate_compact_graph',
+                      action='store_true', default=False,
+                      help="Draw a simplified graph omitting binary package links.")
     options, args = parser.parse_args()
 
     config_paths = None
@@ -725,6 +793,14 @@ def main():
     target_config = TargetConfig(options.architecture,
                                  options.variant)
     target_config.InitializeFromConfigs(config_paths)
+    if options.generate_compact_graph:
+        options.generate_graph = True
+    
+    if options.generate_graph:
+        # Don't do real builds if generating graphs
+        # this doesn't really work - it prints stuff after graph
+        # options.sequence = True
+        print '/*xdeb dot file starts here'
 
     if options.debug:
         print 'Configuration is:\n%s' % target_config
@@ -776,7 +852,22 @@ def main():
 
     explicit_requests.update(args)
     for pkg in args:
-        expand_depends(options, pkg, 0, 0)
+        current_graph = None
+        if options.generate_graph:
+            dot_relationships[pkg] = list()
+            current_graph = pkg
+        expand_depends(current_graph, options, pkg, 0, 0)
+
+    if options.generate_graph:
+        print 'end of xdeb run output - graph starts here */'
+        for pkg in dot_relationships:
+            print 'digraph "%s" {' % pkg
+            print '"%s" [shape=diamond];' % pkg
+            for rel in dot_relationships[pkg]:
+                print rel
+            print '}'
+        print
+        sys.exit(0)
 
     for pkg in args:
         if pkg not in all_srcs:
@@ -796,6 +887,8 @@ def main():
         except tsort.GraphCycleError, e:
             print "Dependency cycle:", e.graph
             sys.exit(1)
+    install_build_depends(options, build_sequence)
+
     for src in build_sequence:
         mark_needs_build(options, src)
     print "Build sequence:",
@@ -824,6 +917,12 @@ def main():
     install_build_depends(options,
                           [src for src in build_sequence if src in real_build])
 
+    if options.debug:
+        print "Installing already built packages"
+    for src in needs_install:
+        install_built_pkg(src, install_version[src], options, 
+                                            options.destdir, False)
+    
     # In --only-explicit mode, native imports have no particular sequencing
     # requirements.
     if options.only_explicit:
